@@ -5,16 +5,14 @@ This module offers a set of operations that user can modify their excel files.
 import os
 import pathlib
 import warnings
+from importlib.resources import files
 from pathlib import Path
 
-try:
-    from importlib.resources import files
-except ImportError:
-    from importlib_resources import files
-
 import openpyxl
+from urllib3 import disable_warnings
 
 from .excel_definition import *
+from .jira_client import *
 from .milestone import *
 from .priority import *
 from .sprint_schedule import *
@@ -29,13 +27,18 @@ __all__ = [
 
 # Currently, the openpyxl package will report an obsolete warning.
 warnings.simplefilter(action="ignore", category=UserWarning)
+# Disable the HTTPS certificate verification warning.
+disable_warnings()
+
+HERE = pathlib.Path(__file__).resolve().parent
+ASSETS = HERE / "assets"
 
 
 def read_excel_file(
     file: "str | Path",
-    excel_definition: Exceldefinition,
+    excel_definition: ExcelDefinition,
     sprint_schedule: SprintScheduleStore,
-) -> tuple:
+) -> tuple[list[str], list[Story]]:
     """
     Read and parse the excel file
 
@@ -43,7 +46,7 @@ def read_excel_file(
         The excel file that you want to read
 
     :parm excel_definition:
-        The excel column definition which is imported from the :py:class:`Exceldefinition`
+        The excel column definition which is imported from the :py:class:`ExcelDefinition`
 
     :parm sprint_schedule:
         The priority mapping for the :py:class:`Milestone` object.
@@ -74,7 +77,7 @@ def read_excel_file(
 
     start_column = "A1"
     end_column = f"{max_column_index}1"
-    columns = []
+    columns: list[str] = []
 
     for cells in sheet[start_column:end_column]:
         if len(columns) > 0:
@@ -99,7 +102,9 @@ def read_excel_file(
         story: Story = storyFactory.create_story()
         for column_index in range(len(row)):
             column = excel_definition_columns[column_index]
-            story.set_value(column[2], column[1], row[column_index].value)
+            if column["name"] is None:
+                continue
+            story.set_value(column["type"], column["name"], row[column_index].value)
 
         story.calc_sprint_schedule(sprint_schedule)
         stories.append(story)
@@ -142,7 +147,7 @@ def output_to_csv_file(
 def output_to_excel_file(
     file: "str | Path",
     stories: "list[Story]",
-    excel_definition: Exceldefinition,
+    excel_definition: ExcelDefinition,
     columns_in_excel: "list[str] | None" = None,
     over_write: bool = True,
 ):
@@ -156,10 +161,10 @@ def output_to_excel_file(
         A list of :py:class:`Story` which need to be wrote to the excel
 
     :parm excel_definition:
-        The excel column definition which is imported from the :py:class:`Exceldefinition`
+        The excel column definition which is imported from the :py:class:`ExcelDefinition`
 
     :parm columns_in_excel:
-        Using separate column names instead of importing from the :py:class:`Exceldefinition`. Usually, it comes from the input excel file.
+        Using separate column names instead of importing from the :py:class:`ExcelDefinition`. Usually, it comes from the input excel file.
 
     :parm over_write:
         Whether or not the exist output file will be over-write.
@@ -180,14 +185,13 @@ def output_to_excel_file(
     wb = openpyxl.Workbook()
     sheet = wb.active
 
+    # TODO: Refactor this method to return needed properties list.
     excel_definition_columns = excel_definition.get_columns()
 
     # Use original excel column name first.
     columns = columns_in_excel
     if columns is None:
-        columns = [
-            column_name for _, column_name, _, _, _, _, _ in excel_definition_columns
-        ]
+        columns = [column["name"] for column in excel_definition_columns]
 
     for column_index in range(len(columns)):
         cell = sheet.cell(row=1, column=column_index + 1)
@@ -197,10 +201,14 @@ def output_to_excel_file(
 
     if stories is not None:
         for row_index in range(len(stories)):
-            for column_index, column_name, _, _, _, _, _ in excel_definition_columns:
-                cell = sheet.cell(row=row_index + 2, column=column_index)
+            for column in excel_definition_columns:
+                if column["name"] is None:
+                    continue
+                cell = sheet.cell(row=row_index + 2, column=column["index"])
                 if hasattr(cell, "value"):
-                    setattr(cell, "value", stories[row_index].format_value(column_name))
+                    setattr(
+                        cell, "value", stories[row_index].format_value(column["name"])
+                    )
 
     wb.save(str(file))
     wb.close()
@@ -241,7 +249,7 @@ def process_excel_file(
         sprint_schedule.load_file(sprint_schedule_config)
         print("Using custom sprint schedule...")
 
-    excel_definition = Exceldefinition()
+    excel_definition = ExcelDefinition()
     if excel_definition_config is None:
         excel_definition.load(
             files("jira_tool.assets").joinpath("excel_definition.json").read_text()
@@ -270,18 +278,76 @@ def process_excel_file(
         print("There are no stories inside the excel file.")
         return
 
-    sort_strategy_priorities = excel_definition.get_sort_strategy_priorities()
+    # TODO: Get Jira Status and filter stories.
+    from dotenv import load_dotenv
 
-    for sort_strategy in sort_strategy_priorities:
-        if sort_strategy.lower() in "InlineWeights".lower():
-            stories = sorted(stories, reverse=True)
-        elif sort_strategy.lower() in "SortOrder".lower():
-            sort_stories_by_property_and_order(stories, excel_definition)
-        elif sort_strategy.lower() in "RaiseRanking".lower():
-            stories = sort_stories_by_raise_ranking(stories, excel_definition)
+    load_dotenv(ASSETS / ".env")
+
+    jira_acccess_token: str | None = os.environ.get("JIRA_ACCESS_TOKEN", default=None)
+    if (
+        # TODO: Avoid duplicate logic.
+        jira_acccess_token is None
+        or jira_acccess_token.isspace()
+        or len(jira_acccess_token) == 0
+    ):
+        print(
+            "The jira access token is invalid. Please use the update-jira-access-token command to add/update token."
+        )
+        return
+
+    jira_client = JiraClient(jira_acccess_token)
+
+    if not jira_client.health_check():
+        print(
+            "The jira access token is revoked. Please use the update-jira-access-token command to add/update token."
+        )
+        return
+
+    # TODO: Move to json file.
+    noneed_sort_statuses = [
+        "SPRINT COMPLETE",
+        "PENDING RELEASE",
+        "PRODUCTION TESTING",
+        "CLOSED",
+    ]
+
+    jira_query_result = jira_client.get_stories_status(
+        [s["storyId"].strip() for s in stories]
+    )
+
+    stories_need_sort = []
+    stories_no_need_sort = []
+
+    for story in stories:
+        story_id: str = story["storyId"].lower().strip()
+        if story_id in jira_query_result:
+            story["status"] = jira_query_result[story_id]
+            if story["status"].upper() in noneed_sort_statuses:
+                story.need_sort = False
+                stories_no_need_sort.append(story)
+                continue
+        stories_need_sort.append(story)
+
+    sort_strategies = excel_definition.get_sort_strategies()
+
+    for sort_strategy in sort_strategies:
+        if sort_strategy["name"] is None:
+            continue
+        if sort_strategy["name"].lower() in "InlineWeights".lower():
+            stories_need_sort = sorted(stories_need_sort, reverse=True)
+        elif sort_strategy["name"].lower() in "SortOrder".lower():
+            sort_stories_by_property_and_order(stories_need_sort, excel_definition)
+        elif sort_strategy["name"].lower() in "RaiseRanking".lower():
+            stories_need_sort = sort_stories_by_raise_ranking(
+                stories_need_sort, excel_definition, sort_strategy["config"]
+            )
 
     output_to_excel_file(
-        output_file, stories, excel_definition, excel_columns, over_write
+        output_file,
+        stories_need_sort + stories_no_need_sort,
+        excel_definition,
+        excel_columns,
+        over_write,
     )
 
     print(f"{output_file} has been saved.")
